<?php

namespace Drupal\fpdi_print\Controller;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Render\RendererInterface as CoreRendererInterface;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\StreamedResponse;
use Symfony\Component\Yaml\Yaml;

/**
 * Controller class for printing Views.
 */
class ViewPrintController extends ControllerBase {

  /**
   * The Entity Type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The current request.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $currentRequest;

  /**
   * The renderer for renderable arrays.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  public $rationPixel2mm = 0.27; // 100px = 0.27mm (75dpi)

  public $view;

  public $DataFields;
  /**
   * {@inheritdoc}
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, Request $current_request, AccountInterface $current_user, CoreRendererInterface $renderer) {
    $this->entityTypeManager = $entity_type_manager;
    $this->currentRequest = $current_request;
    $this->currentUser = $current_user;
    $this->renderer = $renderer;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('request_stack')->getCurrentRequest(),
      $container->get('current_user'),
      $container->get('renderer')
    );
  }

  /**
   * Print an entity to the selected format.
   *
   * @param string $export_type
   *   The export type.
   * @param string $view_name
   *   The view name.
   * @param string $display_id
   *   The view display to render.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   The response object on error otherwise the Print is sent.
   */
  public function viewPrint($view_name, $display_id, $option_id) {
    // Create the Print engine plugin.
    $config = $this->config('fpdi_print.settings');
    /** @var \Drupal\views\Entity\View $view */
    $loadView = $this->entityTypeManager->getStorage('view')->load($view_name);
    $this->view = $loadView->getExecutable();
    $this->view->setDisplay($display_id);
    [$areaType, $optionId] = explode('-', $option_id);
    $option = $this->view->getHandlers($areaType);
    $positions = [];
    if ($args = $this->currentRequest->query->get('view_args')) {
      $this->view->setArguments($args);
    }
    $this->view->execute();

    foreach ($this->view->field as $id => $field) {
      $this->DataFields[$id] = $field->theme(current($this->view->result));
    }
    $allowExtensions = ['png','jpge','jpg','ai','esp','svg'];
    $filePdfTemplate = NULL;
    $filePdfName = null;
    $viewHeader = null;
    $viewFooter = null;
    if(!empty($option[$optionId])){
      $positionText = $option[$optionId]['position_text'];
      $filePdfTemplate = $option[$optionId]['file_pdf'];
      $filePdfName = $option[$optionId]['file_pdf_name'];
      $print_logo = $option[$optionId]['print_logo'];

      if(!empty($positionText)){
        $positionText = Yaml::parse($positionText);
        if(!empty($positionText)){
          $positions = $this->getPosition($positionText);
        }
      }else{
        foreach (['header', 'footer', 'empty'] as $area_type) {
          $handlers = &$this->view->display_handler->getHandlers($area_type);
          if(!empty($handlers)){
            foreach ($handlers as $idHandler=>$valHandler){
              if($valHandler->field == 'area_fpdi_print_views'){
                unset($handlers[$idHandler]);
                continue;
              }
              $viewsHeaderFooter[$area_type][] = $handlers[$idHandler];
              unset($handlers[$idHandler]);
            }
          }
        }
        $renderView = $this->view->render();
        $positions[1] = [
          ['html' => (string) \Drupal::service('renderer')->render($renderView)]
        ];
        if(!empty( $viewsHeaderFooter['header'])){
          foreach ($viewsHeaderFooter['header'] as $header){
            $render = $header->render();
            $viewHeader .= (string)\Drupal::service('renderer')->render($render);
          }
        }
        if ($print_logo) {
          $site_config = \Drupal::config('system.site');
          $logo =  \Drupal::root().file_url_transform_relative(file_create_url(theme_get_setting('logo.url')));
          $path_parts = pathinfo($logo);
          $logo_width_max = 10;
          $logo_height_max = 11;
          if(in_array(strtolower($path_parts['extension']),$allowExtensions)){
            if($path_parts['extension'] != 'svg'){
              [$logo_width,$logo_height] = getimagesize($logo);
              $logo_width_max = ceil($logo_height_max * $logo_width / $logo_height);
            }
          }else{
            $logo = null;
          }
          $Header = [
            'logo' => $logo,
            'width' => $logo_width_max,
            'title' => $site_config->get('name'),
            'string' => $site_config->get('slogan'),
          ];
          if (!empty($viewHeader)) {
            $viewHeader = $Header + ['html' => $viewHeader];
          }
        }
        if(!empty( $viewsHeaderFooter['footer'])){
          foreach ($viewsHeaderFooter['footer'] as $footer){
            $render = $footer->render();
            $viewFooter .= (string)\Drupal::service('renderer')->render($render);
          }
        }
      }
    }

    $pdf = \Drupal::service('fpdi_print.print_builder')->getPDF($positions, $filePdfTemplate, $viewHeader, $viewFooter);
    $response = new StreamedResponse();
    if(!empty($filePdfName)){
      $filePdfName = trim(strip_tags($this->convertTokenValue($filePdfName)));
      $extractPath = explode(DIRECTORY_SEPARATOR, $filePdfName);
      if(count($extractPath)>1){
        $filePdfName = array_pop($extractPath);
        $path = implode(DIRECTORY_SEPARATOR, $extractPath);
        if (!file_exists($path)) {
          $is_created = \Drupal::service('file_system')->mkdir($path);
          if(!$is_created){
            $path = FALSE;
          }
        }
      }
      $pdfOutput = 'D';
      $filePdfName = trim(str_replace(['.pdf','.PDF'],'', $filePdfName)).'.pdf';
      if(!empty($path)){
        $pdfOutput = 'FD';
        $filePdfName = implode(DIRECTORY_SEPARATOR, [$path, $filePdfName]);
      }
      return $response->setCallback(function () use ($pdf, $filePdfName, $pdfOutput) {
        ob_clean();
        $pdf->Output($filePdfName ,$pdfOutput);
      });
    }

    return $response->setCallback(function () use ($pdf) {
      ob_clean();
      $handle = fopen('php://output', 'w+');
      $pdf->Output();
      fclose($handle);
    });
  }

  /**
   * Validate that the current user has access.
   *
   * We need to validate that the user is allowed to access views display id
   *
   * @param string $view_name
   *   The view name.
   * @param string $display_id
   *   The view display to render.
   *
   * @param string $option_id
   *   The option id.
   * @return \Drupal\Core\Access\AccessResult
   *   The access result object.
   */
  public function checkAccess($view_name, $display_id, $option_id = '') {
    $view = $this->entityTypeManager->getStorage('view')->load($view_name)->getExecutable();
    $account = $this->currentUser();

    // Check the content permission.
    $result = AccessResult::allowedIfHasPermission($account, 'access content');

    // Also check the permissions defined by the view.
    return $result->isAllowed() && $view->access($display_id, $account) ? $result : AccessResult::forbidden();
  }

  protected function convertTokenValue($text) {
    $style = $this->view->getStyle();
    $token = $style->tokenizeValue($text, 0);
    $value = $style->globalTokenReplace($token);
    $view_result = $this->view->result;
    if (strpos($text, '{{') !== FALSE && !empty($view_result)) {
      $twig_service = \Drupal::service('twig');
      $value = (string) $twig_service->renderInline($text, $this->DataFields);
    }
    return $value;
  }

  public function getPosition($positionText) {
    $positions = FALSE;
    $pageNoDefault = 1;
    $imageWidthDefault = 80;
    foreach ($positionText as &$position){
      $pageNo = !empty($position['page']) ? $position['page'] : $pageNoDefault;
      if(!empty($position['text'])) {
        $position['text'] = $this->convertTokenValue($position['text']);
        $positions[$pageNo][] = $position;
      } elseif(!empty($position['image'])) {
        $doc = new \DOMDocument();
        @$doc->loadHTML($position['image']);
        $tags = $doc->getElementsByTagName('img');
        //if there is more than 1 image we move next image 80 point
        $widthDefault = !empty($position['width']) ? $position['width'] : $imageWidthDefault;
        if(!empty($tags->length)){
          foreach ($tags as $i=>$tag) {
            $width = !empty($position['width']) ? $position['width'] : $widthDefault = $tag->getAttribute('width');
            $height = !empty($position['height']) ? $position['height'] : $tag->getAttribute('height');
            $positions[$pageNo][] = [
              'image' => $tag->getAttribute('src'),
              'x' => $position['x'] + $i*$widthDefault*$this->rationPixel2mm,
              'y' => $position['y'],
              'width' => $width*$this->rationPixel2mm,
              'height' => $height*$this->rationPixel2mm,
            ];
          }
        }else{
          $positions[$pageNo][] = $position;
        }
      }elseif(!empty($position['html'])){
        $position['html'] = $this->convertTokenValue($position['html']);
        $positions[$pageNo][] = $position;
      }
    }
    return $positions;
  }

}
