<?php

namespace Drupal\fpdi_print;

use Drupal\Component\Utility\Html;
use setasign\Fpdi\Tcpdf\Fpdi;

class Pdf extends Fpdi {

  public $header = [];

  public $footer = [];

  public $headerHTML = '';

  public $headerMarginTop = 8;

  public $footerHTML = '';

  public $footerHeight = 20;

  public $style = '';

  /**
   * Draw an imported PDF logo on every page
   */
  public function Header() {
    if (!empty($this->header)) {
      $headerData = $this->header;
      $headerFont = $this->getHeaderFont();
      if(!empty($this->header['html'])){
        $this->headerHTML .= $this->header['html'];
      }
      if (!empty($this->header['logo'])) {
        $path_parts = pathinfo($headerData['logo']);
        $imgtype = strtolower($path_parts['extension']);
        if (in_array($imgtype, ['eps', 'ai'])) {
          $this->ImageEps($headerData['logo'], '', '', $headerData['width']);
        }
        elseif ($imgtype == 'svg') {
          $this->ImageSVG($headerData['logo'], '', '', $headerData['width']);
        }
        else {
          $this->Image($headerData['logo'], '', $this->headerMarginTop, $headerData['width']);
        }
      }
      $cell_height = $this->getCellHeight($headerFont[2] / $this->k);
      // set starting margin for text data cell
      $header_x = $this->original_lMargin + ($headerData['width'] * 1.1);
      $cw = $this->w - $this->original_lMargin - $this->original_rMargin - ($headerData['width'] * 1.1);
      $this->SetTextColorArray($this->header_text_color);
      // header title
      $this->SetFont($headerFont[0], 'B', $headerFont[2] + 1);
      $this->SetXY($header_x, $this->headerMarginTop);
      if (!empty($headerData['title'])) {
        $this->Cell($headerData['width'], $cell_height, $headerData['title'], 0, 1, '', 0, '', 0);
      }
      // header string
      $this->SetFont($headerFont[0], $headerFont[1], $headerFont[2]);
      $this->SetX($header_x);
      $this->MultiCell($cw, $cell_height, $headerData['string'], 0, '', 0, 1, '', '', TRUE, 0, FALSE, TRUE, 0, 'T', FALSE);
    }
    if (!empty($this->headerHTML)) {
      if (!empty($this->style)) {
        $this->headerHTML .= $this->style;
      }
      $this->writeHTML($this->headerHTML, FALSE, 0, TRUE, 0);
    }
  }

  public function Footer()
  {
    if(!empty($this->footerHTML)) {
      if(!empty($this->style)){
        $this->footerHTML .= $this->style;
      }
      $this->writeHTMLCell(null,$this->footerHeight, null ,-$this->footerHeight,$this->footerHTML);
    }
    $totalPage = $this->getAliasNbPages();
    $currentPage = $this->getAliasNumPage();
    if ($totalPage > 1 && $currentPage > 1) {
      $page = t('Page') . ' ' . $currentPage . '/' . $totalPage;
      $this->SetY(-10);
      $this->Cell(0, 5, $page, FALSE, FALSE, 'R');
    }
  }
}
/**
 * The print builder service.
 */
class PrintBuilder {

  /*
   * array x, y coordinate positions in the page, the page number starts with 1
   * @param array $positions
   *
   * pdf template file path or fid
   * @param string $filePdfTemplate
   *
   * define array header ['logo'|'width'|'title'|'string'|'text_color'|'line_color']
   * @param array $header
   *
   * define array footer ['text_color'|'line_color']
   * @param array $footer
   *
   * @return \setasign\Fpdi\Tcpdf\Fpdi
   */
  public function getPDF($positions = '', $filePdfTemplate = '', $header = FALSE, $footer = FALSE) {
    $config = \Drupal::config('fpdi_print.settings');
    $pdf = new Pdf();
    if(!empty($header) && is_string($header)){
      $pdf->headerHTML = $header;
    }
    if(!empty($footer) && is_string($footer)){
      $pdf->footerHTML = $footer;
      if(!empty($config->get('footer_height'))){
        $pdf->footerHeight = $config->get('footer_height');
      }
    }
    if(!empty($header) && is_array($header)){
      $pdf->header = $header;
    }else{
      $pdf->setPrintHeader($header);
    }
    if(is_array($footer)){
      $pdf->footer = $footer;
    }else{
      $pdf->setPrintFooter($footer);
    }
    $currentUser = \Drupal::currentUser();
    if(!empty($currentUser)){
      $pdf->SetAuthor($currentUser->getDisplayName());
    }

    $pageCount = 1;

    $hasTemplate = FALSE;

    //if file pdf template is fid
    if (!empty($filePdfTemplate) && is_numeric($filePdfTemplate)) {
      $filePdfTemplate = $this->getRealPathFromFid($filePdfTemplate);
    }
    if (!empty($filePdfTemplate) && file_exists($filePdfTemplate)) {
      $pageCount = $pdf->setSourceFile($filePdfTemplate);
      $hasTemplate = TRUE;
    }
    foreach (range(1, $pageCount) as $page) {
      if ($hasTemplate) {
        // import page 1
        $templateId = $pdf->importPage($page);
        // get the size of the imported page
        $size = $pdf->getTemplateSize($templateId);

        // create a page (landscape or portrait depending on the imported page size)
        if ($size[0] > $size[1]) {
          $pdf->AddPage('L', [$size[0], $size[1]]);
        }
        else {
          $pdf->AddPage('P', [$size[0], $size[1]]);
        }
        // use the imported page
        $pdf->useTemplate($templateId);
      }else{
        $pdf->AddPage($config->get('orientation'),$config->get('page_format'));
      }
      if(!empty($config->get('default_css'))){
        $pdf->style = '<style>'.$this->getStyle($config->get('default_css')).'</style>';
      }
      if (!empty($positions[$page])) {
        foreach ($positions[$page] as $position) {
          if (!empty(array_intersect(array_keys($position), ["text","image","html",]))) {
            if(empty($position['x'])) {
              $position['x'] = 0;
            }
            if(empty($position['y'])) {
              $position['y'] = 0;
            }
            if (!empty($position['text'])) {
              $position['text'] = $this->formatText($position['text']);
              $pdf->SetXY($position['x'], $position['y']);
              if (!empty($position['height'])) {
                $pdf->MultiCell(0, $position['height'], $position['text']);
              }
              else {
                $pdf->Write(0, $position['text']);
              }
            }
            if (!empty($position['image'])) {
              $position['image'] = $this->formatImage($position['image']);
              $path_parts = pathinfo($position['image']);
              $imgType = strtolower($path_parts['extension']);
              $img_width = '';
              if (!empty($position['width'])) {
                $img_width = $position['width'];
              }
              if (in_array($imgType,['eps','ai'])) {
                $this->ImageEps($position['image'], $position['x'], $position['y'], $img_width);
              } elseif ($imgType == 'svg') {
                $this->ImageSVG($position['image'], $position['x'], $position['y'], $img_width);
              } else {
                [$img_width, $img_height] = getimagesize($position['image']);
                if (!empty($position['width'])) {
                  $img_width = $position['width'];
                  $img_height = $position['height'];
                }
                $pdf->Image($position['image'], $position['x'], $position['y'], $img_width, $img_height, $imgType);
              }
            }
            if (!empty($position['html'])) {
              $pdf->SetXY($position['x'], $position['y']);
              if (!empty($position['height'])) {
                if(!empty($pdf->style)){
                  $position['text'] .= $pdf->style;
                }
                $pdf->MultiCell(0, $position['height'], $pdf->WriteHTML($position['text']));
              }
              else {
                if(!empty($pdf->style)){
                  $position['html'] = $pdf->style.$position['html'];
                }
                $pdf->writeHTML($position['html'],true, false, true, false, '');
              }
            }
          }
        }
      }

    }
    return $pdf;
  }

  /**
   * filter value
   */
  protected function formatText($value) {
    $value = Html::decodeEntities($value);
    $value = preg_replace('/<br(\s+\/)/', "\n", $value);
    $value = trim(strip_tags($value));
    return $value;
  }

  /**
   * get image path
   */
  protected function formatImage($img) {
    $img = Html::decodeEntities($img);
    $img = trim(strip_tags($img, '<img>'));
    $doc = new \DOMDocument();
    $doc->loadHTML($img);
    $xpath = new \DOMXPath($doc);
    $src = $xpath->evaluate("string(//img/@src)");
    if (!empty($src)) {
      $img = $src;
    }
    $filename = $this->getPath($img);
    $is_picture = is_file($filename) && is_readable($filename) && getimagesize($filename);
    if ($is_picture) {
      return $filename;
    }
    return FALSE;
  }
  protected function hex2rgb ( $hex_color ) {
    $values = str_replace( '#', '', $hex_color );
    switch ( strlen( $values ) ) {
      case 3;
        [ $r, $g, $b ] = sscanf( $values, "%1s%1s%1s" );
        return [ hexdec( "$r$r" ), hexdec( "$g$g" ), hexdec( "$b$b" ) ];
      case 6;
        return array_map( 'hexdec', sscanf( $values, "%2s%2s%2s" ) );
      default:
        return false;
    }
  }
  protected function getRealPathFromFid(int $fid) {
    $file = \Drupal\file\Entity\File::load($fid);
    if(!empty($file)){
      $uri = $file->getFileUri();
      $stream_manager = \Drupal::service('stream_wrapper_manager')->getViaUri($uri);
      return $stream_manager->realpath();
    }
    return FALSE;
  }
  /*
   * convert url to patch
   */
  public function getPath($url){
    $filename = $url;
    if (is_numeric($url)) {
      $filename = $this->getRealPathFromFid($url);
    } elseif (!file_exists($url)) {
      //remove url absolute
      $urlSite = \Drupal\Core\Url::fromUserInput('/', ['absolute' => TRUE])
        ->toString();
      if (strpos($url, $urlSite) !== false) {
        $url = str_replace($urlSite, '', $url);
        $urlParse = parse_url($url);
        //Convert url to path
        $filename = \Drupal::root() . $urlParse['path'];
      }
    }
    return $filename;
  }

  protected function getStyle($css) {
    $filename = $this->getPath($css);
    return file_get_contents($filename);
  }

}
